// Precip.cpp: implementation of the CPrecip class.
//
//////////////////////////////////////////////////////////////////////

#include <cstddef>

#include "Precip.h"
#include "WeatherReport.h"
#include "Climate.h"
#include "Dice.h"

string CPrecip::m_strNames[] = {
    "None",
    "Sprinkle",
    "Shower",
    "Heavy",
    "Steady",
};

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPrecip::CPrecip(int nPrecip) : m_fDuration(0.0), m_fAmount(0.0), m_bStorm(false), m_bExtendedStorm(false), m_bSnow(false)
{
	m_nType = nPrecip;
}

CPrecip::~CPrecip()
{

}

CPrecip& CPrecip::operator=(const CPrecip& rhs)
{
	m_nType = rhs.m_nType;
	m_fDuration = rhs.m_fDuration;
	m_fAmount = rhs.m_fAmount;
	m_bStorm = rhs.m_bStorm;
	m_bExtendedStorm = rhs.m_bExtendedStorm;
	m_bSnow = rhs.m_bSnow;
	return *this;
}

string CPrecip::getName() const
{
	return m_strNames[m_nType];
}

float CPrecip::getAmount() const
{
	return m_fAmount;
}

float CPrecip::getDuration() const
{
	return m_fDuration;
}

bool CPrecip::isStorm() const
{
	return m_bStorm;
}

bool CPrecip::isExtendedStorm() const
{
	return m_bExtendedStorm;
}

bool CPrecip::isSnow() const
{
	return m_bSnow;
}


void CPrecip::updateStats()
{
	CWeatherReport* pReport = CWeatherReport::instance();
	int nWeather = pReport->getWeather();
	int nTemp = pReport->getTemp();
	int nDegrees = pReport->getTempDegrees();
	int nWind = pReport->getWind();

	m_bExtendedStorm = false;
	m_bStorm = false;
	m_bSnow = false;
	m_fAmount = 0.0;
	m_fDuration = 0.0;

	if (nWeather == CWeather::PRECIPITATION)
    {
		// is this a long rain that might last more than one day?
         if (m_nType != STEADY) 
             m_bExtendedStorm = false;
		 else
             m_bExtendedStorm = true;

		 // is it cold enough to snow
         if (nDegrees < 35)
             m_bSnow = true;
         else
             m_bSnow = false;

		// is this a storm?
         if ((nTemp >= CTemperature::WARM) && ((m_nType == SHOWER) || (m_nType == HEAVY)))
             m_bStorm = true;
         else
             m_bStorm = false;

		 // is it a snowstorm or blizzard?
         if (m_bSnow && (m_nType >= HEAVY) && (nWind >= CWind::BLUSTERY))
             m_bStorm = true;

         /* determine duration and m_fAmount of precipitation */
         switch (m_nType) {
         case SPRINKLE :
             m_fDuration = ((float)5 * CDice::roll(6) / 60);
             m_fAmount = ((float)m_fDuration * 0.2);
             break;
         case SHOWER :
             m_fDuration = ((float)10 * CDice::roll(6, 2) / 60);
             m_fAmount = m_fDuration * 0.4;
             break;
         case HEAVY : 
             m_fDuration = (float)(CDice::roll(6) + 1);
             m_fAmount = m_fDuration * 0.6;
             break;
         case STEADY :
             m_fDuration = 24.0;
             m_fAmount = (float)(10 - CDice::roll(6)); 
             break;
         }
         if (m_bStorm)
             m_fAmount *= 2.0;
	}
}

void CPrecip::randomType()
{
	 CWeatherReport* pReport = CWeatherReport::instance();
	 int nWeather = pReport->getWeather();

     m_nType = NONE;

     /* if yesterday had steady precipitation, there is a 1/3 chance */ 
     /* on the second day and 1/6 chance on succeeding days that today */
     /* also has steady precipitation */
	 CWeatherReport* pYesterday = pReport->getYesterday();
     if ((pYesterday->getPrecip() == STEADY) && (CDice::roll(6) <= 2 /* - CWeather::m_bExtendedStorm*/))
     {
         pReport->setWeather(CWeather::PRECIPITATION);
         m_nType = STEADY;
		 // updateStats();
	     return;
     }

     if (nWeather == CWeather::PRECIPITATION)
     {
         int roll = CDice::roll(100);
         m_nType = STEADY;
		 CClimate* pClimate = pReport->getClimate();
		 int nMonth = pReport->getMonth();
         for (int i = STEADY; i >= SPRINKLE; i--)
		 {
             if (roll <= pClimate->getPrecip(nMonth, i))
			 {
                 m_nType = i;
			 }
		 }
     }
	 updateStats();
}

